// tconvlib.h : text converter with XML support
// by Akira Kawamata
// Copyright (c) 1999,2000 by Pie Dey Co.,Ltd.

#ifdef TCONVLIB_EXPORTS
#define TCONVLIB_API __declspec(dllexport)
#else
#define TCONVLIB_API __declspec(dllimport)
#endif

extern "C" {

#ifdef TCONVLIB_EXPORTS
class CTConv;
typedef CTConv * TCONVLIB_HANDLE;
#else
typedef void * TCONVLIB_HANDLE;
#endif

#define TCONVLIB_LINETERM_FROM_CRLF 0x0400
#define TCONVLIB_LINETERM_FROM_CR 0x0200
#define TCONVLIB_LINETERM_FROM_LF 0x0100
#define TCONVLIB_LINETERM_TO_CRLF 0x0004
#define TCONVLIB_LINETERM_TO_CR 0x0002
#define TCONVLIB_LINETERM_TO_LF 0x0001

#define TCONVLIB_LINETERM_TRANSPARENT 0x0000
#define TCONVLIB_LINETERM_CRLF2LF (TCONVLIB_LINETERM_FROM_CRLF|TCONVLIB_LINETERM_TO_LF)
#define TCONVLIB_LINETERM_LF2CRLF (TCONVLIB_LINETERM_FROM_LF|TCONVLIB_LINETERM_TO_CRLF)
#define TCONVLIB_LINETERM_CR2LF   (TCONVLIB_LINETERM_FROM_CR|TCONVLIB_LINETERM_TO_LF)
#define TCONVLIB_LINETERM_LF2CR   (TCONVLIB_LINETERM_FROM_LF|TCONVLIB_LINETERM_TO_CR)
#define TCONVLIB_LINETERM_ALL2LF  (TCONVLIB_LINETERM_FROM_CRLF|TCONVLIB_LINETERM_FROM_CR|TCONVLIB_LINETERM_FROM_LF|TCONVLIB_LINETERM_TO_LF)

#define TCONVLIB_ERRCLASS_FATAL 1
#define TCONVLIB_ERRCLASS_WARN 2
#define TCONVLIB_ERRCLASS_TO_UNICODE 3
#define TCONVLIB_ERRCLASS_FROM_UNICODE 4

typedef BOOL (CALLBACK* TConvLib_Read_Callback)( TCONVLIB_HANDLE handle,
	LPBYTE  lpBuffer,
	DWORD   dwBufferSize,
	LPDWORD lpdwFilled,
	void * lpData );

typedef BOOL (CALLBACK* TConvLib_Error_Callback)( TCONVLIB_HANDLE handle,
	LPCWSTR lpMessage,
	DWORD   dwClass,
	const LPBYTE lpLostCharacter,
	DWORD   dwLostCharacterSize,
	LPWSTR  lpAlternateStringBuffer,
	DWORD   dwAlternateStringSize,
	LPDWORD lpdwAlternateStringFilled,
	void *  lpData );

TCONVLIB_API TCONVLIB_HANDLE TConvLib_Create( 
	LPCWSTR lpctInputEncoding,
	LPCWSTR lpctOutputEncoding,
	LPCWSTR lpctInputEncodingDefault,
	LPCWSTR lpctXMLEncoding,
	DWORD   dwFlags,
	TConvLib_Read_Callback  lpfReadCallBack,
	TConvLib_Error_Callback lpfErrorCallBack,
	void * lpData );

TCONVLIB_API BOOL TConvLib_ReadNext( TCONVLIB_HANDLE handle,
	LPBYTE  lpBuffer,
	DWORD   dwBufferSize,
	LPDWORD lpdwFilled );

TCONVLIB_API BOOL TConvLib_Delete( TCONVLIB_HANDLE handle );

TCONVLIB_API BOOL TConvLib_IsEOF( TCONVLIB_HANDLE handle );

}

#ifdef TCONVLIB_SUPPORT_CLASS

inline BOOL CALLBACK TConvLib_Support_Class_Read_Callback( TCONVLIB_HANDLE handle,
	LPBYTE  lpBuffer,
	DWORD   dwBufferSize,
	LPDWORD lpdwFilled,
	void * lpData );

inline BOOL CALLBACK TConvLib_Support_Class_Error_Callback( TCONVLIB_HANDLE handle,
	LPCWSTR lpMessage,
	DWORD   dwClass,
	const LPBYTE lpLostCharacter,
	DWORD   dwLostCharacterSize,
	LPWSTR  lpAlternateStringBuffer,
	DWORD   dwAlternateStringSize,
	LPDWORD lpdwAlternateStringFilled,
	void *  lpData );

class CTConvLib
{
	TCONVLIB_HANDLE m_handle;
	LPWSTR m_pErrorMessage;
	BOOL m_bHasWarning;
	BOOL m_UndefinedCodeFatal;
public:
	void setUndefinedCodeFatal( BOOL b ) { m_UndefinedCodeFatal = b; }
	BOOL getUndefinedCodeFatal() { return m_UndefinedCodeFatal; }
	void clearHasWarning( BOOL b ) { m_bHasWarning = FALSE; }
	BOOL hasWarning() { return m_bHasWarning; }
	LPCWSTR derefErrorMessage() {
		if( NULL != m_pErrorMessage ) {
			return m_pErrorMessage;
		}
		return L"";
	}
	virtual BOOL TConvLib_Read_Callback( LPBYTE lpBuffer, DWORD dwBufferSize, LPDWORD lpdwFilled )=0;
	// This is a default Error Handler. It's overridable.
	virtual BOOL TConvLib_Error_Callback( LPCWSTR lpMessage, DWORD dwClass, const LPBYTE lpLostCharacter,
		DWORD dwLostCharacterSize, LPWSTR lpAlternateStringBuffer, DWORD dwAlternateStringSize,
		LPDWORD lpdwAlternateStringFilled )
	{
		delete m_pErrorMessage;
		m_pErrorMessage = new WCHAR[wcslen(lpMessage)+1];
		if( NULL == m_pErrorMessage ) return TRUE;
		wcscpy( m_pErrorMessage, lpMessage );

		switch( dwClass ) {
		case TCONVLIB_ERRCLASS_FATAL:
			return TRUE;
		case TCONVLIB_ERRCLASS_WARN:
			m_bHasWarning = TRUE;
			break;
		case TCONVLIB_ERRCLASS_TO_UNICODE:
			if( m_UndefinedCodeFatal ) {
				return TRUE;
			}
			*lpAlternateStringBuffer = L'?';
			*lpdwAlternateStringFilled = 1;
			m_bHasWarning = TRUE;
			break;
		case TCONVLIB_ERRCLASS_FROM_UNICODE:
			if( m_UndefinedCodeFatal ) {
				return TRUE;
			}
			*lpAlternateStringBuffer = L'?';
			*lpdwAlternateStringFilled = 1;
			m_bHasWarning = TRUE;
			break;
		}
		return FALSE;
	}
	BOOL TConvLib_Create( LPCWSTR lpctInputEncoding=NULL, LPCWSTR lpctOutputEncoding=NULL,
		LPCWSTR lpctInputEncodingDefault=NULL, LPCWSTR lpctXMLEncoding=NULL, DWORD dwFlags=0 ) {
		if( NULL != m_handle  ) {
			return FALSE;
		}
		m_handle = ::TConvLib_Create( lpctInputEncoding, lpctOutputEncoding, lpctInputEncodingDefault, 
			lpctXMLEncoding, dwFlags,
			TConvLib_Support_Class_Read_Callback,
			TConvLib_Support_Class_Error_Callback,
			(void *)this );
		return NULL != m_handle;
	}
	BOOL TConvLib_ReadNext( LPBYTE lpBuffer, DWORD dwBufferSize, LPDWORD lpdwFilled ) {
		if( NULL != m_handle  ) {
			return ::TConvLib_ReadNext( m_handle, lpBuffer, dwBufferSize, lpdwFilled );
		}
		return FALSE;
	}
	BOOL TConvLib_Delete() {
		if( NULL != m_handle  ) {
			return ::TConvLib_Delete( m_handle );
		}
		return FALSE;
	}
	BOOL TConvLib_IsEOF() {
		if( NULL != m_handle  ) {
			return ::TConvLib_IsEOF( m_handle );
		}
		return TRUE;
	}
	CTConvLib() {
		m_handle = NULL;
		m_pErrorMessage = NULL;
		m_bHasWarning = FALSE;
		m_UndefinedCodeFatal = FALSE;
	}
	virtual ~CTConvLib() {
		TConvLib_Delete();
		delete m_pErrorMessage;
	}
};

class CTConvLibFromMemory : public CTConvLib
{
	const BYTE * m_pTarget;
	DWORD m_leftBytes;
public:
	BOOL TConvLib_Read_Callback( LPBYTE lpBuffer, DWORD dwBufferSize, LPDWORD lpdwFilled ) {
		DWORD len = min(dwBufferSize,m_leftBytes);
		memcpy( lpBuffer, m_pTarget, len );
		*lpdwFilled = len;
		m_pTarget += len;
		m_leftBytes -= len;
		return TRUE;
	}
	BOOL TConvLib_Create( const BYTE * target, DWORD sizeBytes, 
		LPCWSTR lpctInputEncoding=NULL, LPCWSTR lpctOutputEncoding=NULL, LPCWSTR lpctInputEncodingDefault=NULL,
		LPCWSTR lpctXMLEncoding=NULL, DWORD dwFlags = 0
	) {
		m_pTarget = target;
		m_leftBytes = sizeBytes;
		BOOL b = CTConvLib::TConvLib_Create( lpctInputEncoding, lpctOutputEncoding, lpctInputEncodingDefault, lpctXMLEncoding, dwFlags );
		return b;
	}
};

class CTConvLibQuickWideToSJ
{
	CTConvLibFromMemory m_conv;
	LPSTR m_result;
public:
	CTConvLibQuickWideToSJ( LPCWSTR target ) {
		m_result = NULL;
		DWORD len = wcslen(target);
		BOOL b = m_conv.TConvLib_Create( (const BYTE *)target, (len+1)*sizeof(WCHAR), L"UTF-16LE", L"Shift_JIS" );
		if( b == FALSE ) {
			return;
		}
		DWORD resultLength = len*2+1;
		m_result = new CHAR[resultLength];
		DWORD filled = 0;
		/*BOOL*/ b = m_conv.TConvLib_ReadNext( (LPBYTE)m_result, resultLength, &filled );
		if( b == FALSE ) {
			delete m_result;
			m_result = NULL;
			return;
		}
	}
	virtual ~CTConvLibQuickWideToSJ() {
		delete m_result;
	}
	LPCSTR derefResult() {
		if( NULL != m_result ) {
			return m_result;
		}
		return "?";
	}
};

inline BOOL CALLBACK TConvLib_Support_Class_Read_Callback( TCONVLIB_HANDLE handle,
	LPBYTE  lpBuffer, DWORD dwBufferSize, LPDWORD lpdwFilled, void * lpData ) 
{
	return ((CTConvLib *)lpData)->TConvLib_Read_Callback( lpBuffer, dwBufferSize, lpdwFilled );
}

inline BOOL CALLBACK TConvLib_Support_Class_Error_Callback( TCONVLIB_HANDLE handle,
	LPCWSTR lpMessage,
	DWORD   dwClass,
	const LPBYTE lpLostCharacter,
	DWORD   dwLostCharacterSize,
	LPWSTR  lpAlternateStringBuffer,
	DWORD   dwAlternateStringSize,
	LPDWORD lpdwAlternateStringFilled,
	void *  lpData )
{
	return ((CTConvLib *)lpData)->TConvLib_Error_Callback( lpMessage, dwClass, lpLostCharacter, dwLostCharacterSize,
		lpAlternateStringBuffer, dwAlternateStringSize, lpdwAlternateStringFilled );
}

#endif	/* TCONVLIB_SUPPORT_CLASS */

// end of tconvlib.h
